/*
 * Copyright (C) 2011, Hewlett-Packard Development Company, L.P.
 *   Author: Sebastian Dröge <sebastian.droege@collabora.co.uk>, Collabora Ltd.
 * Copyright (c) 2013-2014, NVIDIA CORPORATION.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation
 * version 2.1 of the License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301 USA
 *
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gst/gst.h>

#include "gstomxh264enc.h"

GST_DEBUG_CATEGORY_STATIC (gst_omx_h264_enc_debug_category);
#define GST_CAT_DEFAULT gst_omx_h264_enc_debug_category

/* prototypes */
static gboolean gst_omx_h264_enc_set_format (GstOMXVideoEnc * enc,
    GstOMXPort * port, GstVideoCodecState * state);
static GstCaps *gst_omx_h264_enc_get_caps (GstOMXVideoEnc * enc,
    GstOMXPort * port, GstVideoCodecState * state);
static GstFlowReturn gst_omx_h264_enc_handle_output_frame (GstOMXVideoEnc *
    self, GstOMXPort * port, GstOMXBuffer * buf, GstVideoCodecFrame * frame);

enum
{
  PROP_0
};

/* class initialization */

#define DEBUG_INIT \
  GST_DEBUG_CATEGORY_INIT (gst_omx_h264_enc_debug_category, "omxh264enc", 0, \
      "debug category for gst-omx video encoder base class");

G_DEFINE_TYPE_WITH_CODE (GstOMXH264Enc, gst_omx_h264_enc,
    GST_TYPE_OMX_VIDEO_ENC, DEBUG_INIT);

static void
gst_omx_h264_enc_class_init (GstOMXH264EncClass * klass)
{
  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
  GstOMXVideoEncClass *videoenc_class = GST_OMX_VIDEO_ENC_CLASS (klass);

  videoenc_class->set_format = GST_DEBUG_FUNCPTR (gst_omx_h264_enc_set_format);
  videoenc_class->get_caps = GST_DEBUG_FUNCPTR (gst_omx_h264_enc_get_caps);

  videoenc_class->cdata.default_src_template_caps = "video/x-h264, "
      "width=(int) [ 16, 4096 ], " "height=(int) [ 16, 4096 ], "
      "stream-format=(string) { byte-stream, avc }, " "alignment=(string) au ";
  videoenc_class->handle_output_frame =
      GST_DEBUG_FUNCPTR (gst_omx_h264_enc_handle_output_frame);

  gst_element_class_set_static_metadata (element_class,
      "OpenMAX H.264 Video Encoder",
      "Codec/Encoder/Video",
      "Encode H.264 video streams",
      "Sebastian Dröge <sebastian.droege@collabora.co.uk>");

  gst_omx_set_default_role (&videoenc_class->cdata, "video_encoder.avc");
}

static void
gst_omx_h264_enc_init (GstOMXH264Enc * self)
{
}

static gboolean
gst_omx_h264_enc_set_format (GstOMXVideoEnc * enc, GstOMXPort * port,
    GstVideoCodecState * state)
{
  GstOMXH264Enc *self = GST_OMX_H264_ENC (enc);
  GstCaps *peercaps;
  OMX_PARAM_PORTDEFINITIONTYPE port_def;
  OMX_VIDEO_PARAM_PROFILELEVELTYPE param;
  OMX_VIDEO_CONFIG_NALSIZE OMXNalSize;
  OMX_ERRORTYPE err;
  const gchar *profile_string, *level_string;
  const gchar *out_format = NULL;

  gst_omx_port_get_port_definition (GST_OMX_VIDEO_ENC (self)->enc_out_port,
      &port_def);
  port_def.format.video.eCompressionFormat = OMX_VIDEO_CodingAVC;
  err =
      gst_omx_port_update_port_definition (GST_OMX_VIDEO_ENC
      (self)->enc_out_port, &port_def);
  if (err != OMX_ErrorNone)
    return FALSE;

  GST_OMX_INIT_STRUCT (&param);
  param.nPortIndex = GST_OMX_VIDEO_ENC (self)->enc_out_port->index;

  err =
      gst_omx_component_get_parameter (GST_OMX_VIDEO_ENC (self)->enc,
      OMX_IndexParamVideoProfileLevelCurrent, &param);
  if (err != OMX_ErrorNone) {
    GST_WARNING_OBJECT (self,
        "Setting profile/level not supported by component");
  }

  peercaps = gst_pad_peer_query_caps (GST_VIDEO_ENCODER_SRC_PAD (enc),
      gst_pad_get_pad_template_caps (GST_VIDEO_ENCODER_SRC_PAD (enc)));
  if (peercaps) {
    GstStructure *s;

    if (gst_caps_is_empty (peercaps)) {
      gst_caps_unref (peercaps);
      GST_ERROR_OBJECT (self, "Empty caps");
      return FALSE;
    }

    s = gst_caps_get_structure (peercaps, 0);

    if (err == OMX_ErrorNone) {
      profile_string = gst_structure_get_string (s, "profile");
      if (profile_string) {
        if (g_str_equal (profile_string, "baseline")) {
          param.eProfile = OMX_VIDEO_AVCProfileBaseline;
        } else if (g_str_equal (profile_string, "main")) {
          param.eProfile = OMX_VIDEO_AVCProfileMain;
        } else if (g_str_equal (profile_string, "extended")) {
          param.eProfile = OMX_VIDEO_AVCProfileExtended;
        } else if (g_str_equal (profile_string, "high")) {
          param.eProfile = OMX_VIDEO_AVCProfileHigh;
        } else if (g_str_equal (profile_string, "high-10")) {
          param.eProfile = OMX_VIDEO_AVCProfileHigh10;
        } else if (g_str_equal (profile_string, "high-4:2:2")) {
          param.eProfile = OMX_VIDEO_AVCProfileHigh422;
        } else if (g_str_equal (profile_string, "high-4:4:4")) {
          param.eProfile = OMX_VIDEO_AVCProfileHigh444;
        } else {
          goto unsupported_profile;
        }
      }
      level_string = gst_structure_get_string (s, "level");
      if (level_string) {
        if (g_str_equal (level_string, "1")) {
          param.eLevel = OMX_VIDEO_AVCLevel1;
        } else if (g_str_equal (level_string, "1b")) {
          param.eLevel = OMX_VIDEO_AVCLevel1b;
        } else if (g_str_equal (level_string, "1.1")) {
          param.eLevel = OMX_VIDEO_AVCLevel11;
        } else if (g_str_equal (level_string, "1.2")) {
          param.eLevel = OMX_VIDEO_AVCLevel12;
        } else if (g_str_equal (level_string, "1.3")) {
          param.eLevel = OMX_VIDEO_AVCLevel13;
        } else if (g_str_equal (level_string, "2")) {
          param.eLevel = OMX_VIDEO_AVCLevel2;
        } else if (g_str_equal (level_string, "2.1")) {
          param.eLevel = OMX_VIDEO_AVCLevel21;
        } else if (g_str_equal (level_string, "2.2")) {
          param.eLevel = OMX_VIDEO_AVCLevel22;
        } else if (g_str_equal (level_string, "3")) {
          param.eLevel = OMX_VIDEO_AVCLevel3;
        } else if (g_str_equal (level_string, "3.1")) {
          param.eLevel = OMX_VIDEO_AVCLevel31;
        } else if (g_str_equal (level_string, "3.2")) {
          param.eLevel = OMX_VIDEO_AVCLevel32;
        } else if (g_str_equal (level_string, "4")) {
          param.eLevel = OMX_VIDEO_AVCLevel4;
        } else if (g_str_equal (level_string, "4.1")) {
          param.eLevel = OMX_VIDEO_AVCLevel41;
        } else if (g_str_equal (level_string, "4.2")) {
          param.eLevel = OMX_VIDEO_AVCLevel42;
        } else if (g_str_equal (level_string, "5")) {
          param.eLevel = OMX_VIDEO_AVCLevel5;
        } else if (g_str_equal (level_string, "5.1")) {
          param.eLevel = OMX_VIDEO_AVCLevel51;
        } else {
          goto unsupported_level;
        }
      }

      err =
          gst_omx_component_set_parameter (GST_OMX_VIDEO_ENC (self)->enc,
          OMX_IndexParamVideoProfileLevelCurrent, &param);
      if (err == OMX_ErrorUnsupportedIndex) {
        GST_WARNING_OBJECT (self,
            "Setting profile/level not supported by component");
      } else if (err != OMX_ErrorNone) {
        GST_ERROR_OBJECT (self,
            "Error setting profile %u and level %u: %s (0x%08x)",
            (guint) param.eProfile, (guint) param.eLevel,
            gst_omx_error_to_string (err), err);
        return FALSE;
      }
    }

    out_format = gst_structure_get_string (s, "stream-format");
    if (out_format) {
      if (g_str_equal (out_format, "avc")) {
        self->stream_format = H264_AVC;
      } else if (g_str_equal (out_format, "byte-stream")) {
        self->stream_format = H264_BTS;
      } else {
        goto unsupported_out_format;
      }
    }
    gst_caps_unref (peercaps);
  }

  if (self->stream_format == H264_AVC) {
    GST_OMX_INIT_STRUCT (&OMXNalSize);
    err =
        gst_omx_component_get_config (GST_OMX_VIDEO_ENC (self)->enc,
        OMX_IndexConfigVideoNalSize, &OMXNalSize);
    if (err == OMX_ErrorNone) {
      OMXNalSize.nNaluBytes = 4;
      OMXNalSize.nPortIndex = GST_OMX_VIDEO_ENC (self)->enc_out_port->index;
      err =
          gst_omx_component_set_config (GST_OMX_VIDEO_ENC (self)->enc,
          OMX_IndexConfigVideoNalSize, &OMXNalSize);
      if (err != OMX_ErrorNone) {
        return FALSE;
      }
    }
  }

  return TRUE;

unsupported_profile:
  GST_ERROR_OBJECT (self, "Unsupported profile %s", profile_string);
  gst_caps_unref (peercaps);
  return FALSE;

unsupported_level:
  GST_ERROR_OBJECT (self, "Unsupported level %s", level_string);
  gst_caps_unref (peercaps);
  return FALSE;

unsupported_out_format:
  GST_ERROR_OBJECT (self, "Unsupported stream-format %s", out_format);
  gst_caps_unref (peercaps);
  return FALSE;
}

static GstCaps *
gst_omx_h264_enc_get_caps (GstOMXVideoEnc * enc, GstOMXPort * port,
    GstVideoCodecState * state)
{
  GstOMXH264Enc *self = GST_OMX_H264_ENC (enc);
  GstCaps *caps;
  OMX_ERRORTYPE err;
  OMX_VIDEO_PARAM_PROFILELEVELTYPE param;
  const gchar *profile, *level, *out_format;

  caps = gst_caps_new_simple ("video/x-h264",
      "alignment", G_TYPE_STRING, "au", NULL);

  GST_OMX_INIT_STRUCT (&param);
  param.nPortIndex = GST_OMX_VIDEO_ENC (self)->enc_out_port->index;

  err =
      gst_omx_component_get_parameter (GST_OMX_VIDEO_ENC (self)->enc,
      OMX_IndexParamVideoProfileLevelCurrent, &param);
  if (err != OMX_ErrorNone && err != OMX_ErrorUnsupportedIndex
      && err != OMX_ErrorNotImplemented)
    return NULL;

  if (err == OMX_ErrorNone) {
    switch (param.eProfile) {
      case OMX_VIDEO_AVCProfileBaseline:
        profile = "baseline";
        break;
      case OMX_VIDEO_AVCProfileMain:
        profile = "main";
        break;
      case OMX_VIDEO_AVCProfileExtended:
        profile = "extended";
        break;
      case OMX_VIDEO_AVCProfileHigh:
        profile = "high";
        break;
      case OMX_VIDEO_AVCProfileHigh10:
        profile = "high-10";
        break;
      case OMX_VIDEO_AVCProfileHigh422:
        profile = "high-4:2:2";
        break;
      case OMX_VIDEO_AVCProfileHigh444:
        profile = "high-4:4:4";
        break;
      default:
        g_assert_not_reached ();
        return NULL;
    }

    switch (param.eLevel) {
      case OMX_VIDEO_AVCLevel1:
        level = "1";
        break;
      case OMX_VIDEO_AVCLevel1b:
        level = "1b";
        break;
      case OMX_VIDEO_AVCLevel11:
        level = "1.1";
        break;
      case OMX_VIDEO_AVCLevel12:
        level = "1.2";
        break;
      case OMX_VIDEO_AVCLevel13:
        level = "1.3";
        break;
      case OMX_VIDEO_AVCLevel2:
        level = "2";
        break;
      case OMX_VIDEO_AVCLevel21:
        level = "2.1";
        break;
      case OMX_VIDEO_AVCLevel22:
        level = "2.2";
        break;
      case OMX_VIDEO_AVCLevel3:
        level = "3";
        break;
      case OMX_VIDEO_AVCLevel31:
        level = "3.1";
        break;
      case OMX_VIDEO_AVCLevel32:
        level = "3.2";
        break;
      case OMX_VIDEO_AVCLevel4:
        level = "4";
        break;
      case OMX_VIDEO_AVCLevel41:
        level = "4.1";
        break;
      case OMX_VIDEO_AVCLevel42:
        level = "4.2";
        break;
      case OMX_VIDEO_AVCLevel5:
        level = "5";
        break;
      case OMX_VIDEO_AVCLevel51:
        level = "5.1";
        break;
      default:
        g_assert_not_reached ();
        return NULL;
    }

    gst_caps_set_simple (caps,
        "profile", G_TYPE_STRING, profile, "level", G_TYPE_STRING, level, NULL);
  }

  switch (self->stream_format) {
    case H264_AVC:
      out_format = "avc";
      break;
    case H264_BTS:
      out_format = "byte-stream";
      break;
    default:
      g_assert_not_reached ();
      return NULL;
  }

  gst_caps_set_simple (caps, "stream-format", G_TYPE_STRING, out_format, NULL);

  return caps;
}

static GstFlowReturn
gst_omx_h264_enc_handle_output_frame (GstOMXVideoEnc * self, GstOMXPort * port,
    GstOMXBuffer * buf, GstVideoCodecFrame * frame)
{
  GstOMXH264Enc *h264enc = GST_OMX_H264_ENC (self);

  if (buf->omx_buf->nFlags & OMX_BUFFERFLAG_CODECCONFIG) {
    /* The codec data is SPS/PPS with a startcode => bytestream stream format
     * For bytestream stream format the SPS/PPS is only in-stream and not
     * in the caps!
     */
    if (buf->omx_buf->nFilledLen >= 4 &&
        GST_READ_UINT32_BE (buf->omx_buf->pBuffer +
            buf->omx_buf->nOffset) == 0x00000001) {
      GList *l = NULL;
      GstBuffer *hdrs;
      GstMapInfo map = GST_MAP_INFO_INIT;

      GST_DEBUG_OBJECT (self, "got codecconfig in byte-stream format");
      buf->omx_buf->nFlags &= ~OMX_BUFFERFLAG_CODECCONFIG;

      hdrs = gst_buffer_new_and_alloc (buf->omx_buf->nFilledLen);

      gst_buffer_map (hdrs, &map, GST_MAP_WRITE);
      memcpy (map.data,
          buf->omx_buf->pBuffer + buf->omx_buf->nOffset,
          buf->omx_buf->nFilledLen);
      gst_buffer_unmap (hdrs, &map);
      l = g_list_append (l, hdrs);
      gst_video_encoder_set_headers (GST_VIDEO_ENCODER (self), l);
    }
#ifdef USE_OMX_TARGET_TEGRA
    else {
      if (h264enc->stream_format == H264_AVC) {
        gsize inbuf_size;
        GstBuffer *codec_data_buf;
        GstMapInfo cdmap = GST_MAP_INFO_INIT;
        OMX_U32 length_to_allocate, offset;
        OMX_U8 *str_ptr;
        OMX_U8 *pts_ptr;
        OMX_U8 data[6];
        OMX_U32 *codec_data;
        OMX_U32 sps_length;

        GST_DEBUG_OBJECT (self, "create codec_data caps buffer");

        str_ptr = (buf->omx_buf->pBuffer + buf->omx_buf->nOffset);
        inbuf_size = buf->omx_buf->nFilledLen;

        codec_data =
            (OMX_U32 *) (buf->omx_buf->pBuffer + buf->omx_buf->nOffset);
        sps_length = (codec_data[0] >> 24);

        // Need additional 7 bytes of data to indicate no of sps,pps
        //and how many bytes indicate nal_length,profile etc also we
        //are going to strip down 2 bytes each for the sps and pps
        //length hence reducing the size by 4

        length_to_allocate = (inbuf_size + ADDITIONAL_LENGTH);

        codec_data_buf = gst_buffer_new_and_alloc (length_to_allocate);
        gst_buffer_map (codec_data_buf, &cdmap, GST_MAP_WRITE);

        data[0] = 0x01;
        data[1] = 0x42;         // 66 Baseline profile
        data[2] = 0x40;         // constrained Baseline
        data[3] = 0x15;         // level2.1
        data[4] = 0x03;         // nal length = 4 bytes
        data[5] = 0x01;         // 1SPS

        // append data of 6 bytes and then the sps length and sps data
        memcpy (cdmap.data, data, HEADER_DATA_LENGTH);

        // Strip down the length indicating the NAL data size from 4 bytes to 2 bytes.
        // This is because the down stream qtmux are expecting this to be only 2 bytes

        memcpy ((cdmap.data + HEADER_DATA_LENGTH), (str_ptr + 2),
            DOWNSTREAM_NAL_LENGTH_INDICATOR);

        // Now copy the SPS data
        memcpy ((cdmap.data + HEADER_DATA_LENGTH + 2),
            (str_ptr + ENCODER_NAL_LENGTH_INDICATOR), sps_length);

        offset =
            HEADER_DATA_LENGTH + DOWNSTREAM_NAL_LENGTH_INDICATOR + sps_length;

        pts_ptr = cdmap.data + offset;
        pts_ptr[0] = 1;         // 1 pps

        offset += 1;

        // Strip down the length indicating the NAL data size from 4 bytes to 2 bytes.
        // This is because the down stream qtmux are expecting this to be only 2 bytes
        memcpy ((cdmap.data + offset),
            (str_ptr + (ENCODER_NAL_LENGTH_INDICATOR + sps_length) + 2),
            DOWNSTREAM_NAL_LENGTH_INDICATOR);

        offset += 2;

        // Now copy pps data
        memcpy ((cdmap.data + offset),
            (str_ptr + ((ENCODER_NAL_LENGTH_INDICATOR << 1) + sps_length)),
            (inbuf_size - ((ENCODER_NAL_LENGTH_INDICATOR << 1) + sps_length)));

        gst_buffer_unmap (codec_data_buf, &cdmap);

        self->codec_data = codec_data_buf;
      }
    }
#endif
  }

  return
      GST_OMX_VIDEO_ENC_CLASS
      (gst_omx_h264_enc_parent_class)->handle_output_frame (self, port, buf,
      frame);
}
